@@ -1,17 +1,15 @@
 //! Builder and objects for creating classes in the PHP world.
 
 use crate::errors::{Error, Result};
-use std::{mem, ptr};
+use std::{convert::TryInto, ffi::CString, fmt::Debug, mem};
 
-use crate::{
-    bindings::{
-        zend_class_entry, zend_declare_class_constant, zend_declare_property,
-        zend_register_internal_class_ex,
-    },
-    functions::c_str,
+use crate::bindings::{
+    zend_class_entry, zend_declare_class_constant, zend_declare_property,
+    zend_register_internal_class_ex,
 };
 
 use super::{
+    executor::ExecutorGlobals,
     flags::{ClassFlags, MethodFlags, PropertyFlags},
     function::FunctionEntry,
     types::{
@@ -24,6 +22,98 @@ use super::{
 /// A Zend class entry. Alias.
 pub type ClassEntry = zend_class_entry;
 
+impl PartialEq for ClassEntry {
+    fn eq(&self, other: &Self) -> bool {
+        std::ptr::eq(self, other)
+    }
+}
+
+impl ClassEntry {
+    /// Attempts to find a reference to a class in the global class table.
+    ///
+    /// Returns a reference to the class if found, or [`None`] if the class could
+    /// not be found or the class table has not been initialized.
+    pub fn try_find(name: &str) -> Option<&'static Self> {
+        ExecutorGlobals::get().class_table()?;
+        let name = ZendString::new(name, false).ok()?;
+
+        unsafe {
+            crate::bindings::zend_lookup_class_ex(name.borrow_ptr(), std::ptr::null_mut(), 0)
+                .as_ref()
+        }
+    }
+
+    /// Returns the class flags.
+    pub fn flags(&self) -> ClassFlags {
+        ClassFlags::from_bits_truncate(self.ce_flags)
+    }
+
+    /// Checks if the class is an instance of another class or interface.
+    ///
+    /// # Parameters
+    ///
+    /// * `ce` - The inherited class entry to check.
+    pub fn instance_of(&self, ce: &ClassEntry) -> bool {
+        if self == ce {
+            return true;
+        }
+
+        if ce.flags().contains(ClassFlags::Interface) {
+            let interfaces = match self.interfaces() {
+                Some(interfaces) => interfaces,
+                None => return false,
+            };
+
+            for i in interfaces {
+                if ce == i {
+                    return true;
+                }
+            }
+        } else {
+            loop {
+                let parent = match self.parent() {
+                    Some(parent) => parent,
+                    None => return false,
+                };
+
+                if parent == ce {
+                    return true;
+                }
+            }
+        }
+
+        false
+    }
+
+    /// Returns an iterator of all the interfaces that the class implements. Returns [`None`] if
+    /// the interfaces have not been resolved on the class.
+    pub fn interfaces(&self) -> Option<impl Iterator<Item = &ClassEntry>> {
+        self.flags()
+            .contains(ClassFlags::ResolvedInterfaces)
+            .then(|| unsafe {
+                (0..self.num_interfaces)
+                    .into_iter()
+                    .map(move |i| *self.__bindgen_anon_3.interfaces.offset(i as _))
+                    .filter_map(|ptr| ptr.as_ref())
+            })
+    }
+
+    /// Returns the parent of the class.
+    ///
+    /// If the parent of the class has not been resolved, it attempts to find the parent by name.
+    /// Returns [`None`] if the parent was not resolved and the parent was not able to be found
+    /// by name.
+    pub fn parent(&self) -> Option<&Self> {
+        if self.flags().contains(ClassFlags::ResolvedParent) {
+            unsafe { self.__bindgen_anon_1.parent.as_ref() }
+        } else {
+            let name =
+                unsafe { ZendString::from_ptr(self.__bindgen_anon_1.parent_name, false) }.ok()?;
+            Self::try_find(name.as_str()?)
+        }
+    }
+}
+
 /// Builds a class to be exported as a PHP class.
 pub struct ClassBuilder<'a> {
     name: String,
@@ -43,10 +133,7 @@ impl<'a> ClassBuilder<'a> {
     ///
     /// * `name` - The name of the class.
     #[allow(clippy::unwrap_used)]
-    pub fn new<N>(name: N) -> Self
-    where
-        N: AsRef<str>,
-    {
+    pub fn new<T: Into<String>>(name: T) -> Self {
         // SAFETY: Allocating temporary class entry. Will return a null-ptr if allocation fails,
         // which will cause the program to panic (standard in Rust). Unwrapping is OK - the ptr
         // will either be valid or null.
@@ -57,7 +144,7 @@ impl<'a> ClassBuilder<'a> {
         };
 
         Self {
-            name: name.as_ref().to_string(),
+            name: name.into(),
             ptr,
             extends: None,
             methods: vec![],
@@ -99,16 +186,15 @@ impl<'a> ClassBuilder<'a> {
     /// * `name` - The name of the property to add to the class.
     /// * `default` - The default value of the property.
     /// * `flags` - Flags relating to the property. See [`PropertyFlags`].
-    pub fn property(
+    pub fn property<T: Into<String>>(
         mut self,
-        name: impl AsRef<str>,
+        name: T,
         default: impl IntoZval,
         flags: PropertyFlags,
     ) -> Result<Self> {
         let default = default.as_zval(true)?;
 
-        self.properties
-            .push((name.as_ref().to_string(), default, flags));
+        self.properties.push((name.into(), default, flags));
         Ok(self)
     }
 
@@ -121,10 +207,10 @@ impl<'a> ClassBuilder<'a> {
     ///
     /// * `name` - The name of the constant to add to the class.
     /// * `value` - The value of the constant.
-    pub fn constant(mut self, name: impl AsRef<str>, value: impl IntoZval) -> Result<Self> {
+    pub fn constant<T: Into<String>>(mut self, name: T, value: impl IntoZval) -> Result<Self> {
         let value = value.as_zval(true)?;
 
-        self.constants.push((name.as_ref().to_string(), value));
+        self.constants.push((name.into(), value));
         Ok(self)
     }
 
@@ -146,10 +232,7 @@ impl<'a> ClassBuilder<'a> {
     /// * `T` - The type which will override the Zend object. Must implement [`ZendObjectOverride`]
     /// which can be derived through the [`ZendObjectHandler`](ext_php_rs_derive::ZendObjectHandler)
     /// derive macro.
-    pub fn object_override<T>(mut self) -> Self
-    where
-        T: ZendObjectOverride,
-    {
+    pub fn object_override<T: ZendObjectOverride>(mut self) -> Self {
         self.object_override = Some(T::create_object);
         self
     }
@@ -160,7 +243,7 @@ impl<'a> ClassBuilder<'a> {
     ///
     /// Returns an [`Error`] variant if the class could not be registered.
     pub fn build(mut self) -> Result<&'static mut ClassEntry> {
-        self.ptr.name = ZendString::new_interned(self.name)?.release();
+        self.ptr.name = ZendString::new_interned(&self.name)?.release();
 
         self.methods.push(FunctionEntry::end());
         let func = Box::into_raw(self.methods.into_boxed_slice()) as *const FunctionEntry;
@@ -171,7 +254,7 @@ impl<'a> ClassBuilder<'a> {
                 self.ptr,
                 match self.extends {
                     Some(ptr) => (ptr as *const _) as *mut _,
-                    None => ptr::null_mut(),
+                    None => std::ptr::null_mut(),
                 },
             )
             .as_mut()
@@ -185,7 +268,7 @@ impl<'a> ClassBuilder<'a> {
             unsafe {
                 zend_declare_property(
                     class,
-                    c_str(&name)?,
+                    CString::new(name.as_str())?.as_ptr(),
                     name.len() as _,
                     &mut default,
                     flags.bits() as _,
@@ -195,7 +278,14 @@ impl<'a> ClassBuilder<'a> {
 
         for (name, value) in self.constants {
             let value = Box::into_raw(Box::new(value));
-            unsafe { zend_declare_class_constant(class, c_str(&name)?, name.len() as u64, value) };
+            unsafe {
+                zend_declare_class_constant(
+                    class,
+                    CString::new(name.as_str())?.as_ptr(),
+                    name.len() as u64,
+                    value,
+                )
+            };
         }
 
         if let Some(object_override) = self.object_override {
@@ -205,3 +295,16 @@ impl<'a> ClassBuilder<'a> {
         Ok(class)
     }
 }
+
+impl Debug for ClassEntry {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let name: String = unsafe { ZendString::from_ptr(self.name, false) }
+            .and_then(|str| str.try_into())
+            .map_err(|_| std::fmt::Error)?;
+
+        f.debug_struct("ClassEntry")
+            .field("name", &name)
+            .field("flags", &self.flags())
+            .finish()
+    }
+}
