@@ -2,12 +2,17 @@
 //! determined by a property inside the struct. The content of the Zval is stored in a union.
 
 use core::slice;
-use std::{convert::TryFrom, fmt::Debug, ptr};
+use std::{
+    collections::HashMap,
+    convert::{TryFrom, TryInto},
+    fmt::Debug,
+    ptr,
+};
 
 use crate::{
     bindings::{
-        _call_user_function_impl, _zval_struct__bindgen_ty_1, _zval_struct__bindgen_ty_2,
-        ext_php_rs_zend_string_release, zend_is_callable, zend_resource, zend_value, zval,
+        _zval_struct__bindgen_ty_1, _zval_struct__bindgen_ty_2, ext_php_rs_zend_string_release,
+        zend_is_callable, zend_resource, zend_value, zval,
     },
     errors::{Error, Result},
     php::pack::Pack,
@@ -19,14 +24,14 @@ use crate::php::{
     types::{long::ZendLong, string::ZendString},
 };
 
-use super::{array::ZendHashTable, object::ZendObject};
+use super::{array::ZendHashTable, callable::Callable, object::ZendObject};
 
 /// Zend value. Represents most data types that are in the Zend engine.
 pub type Zval = zval;
 
 impl<'a> Zval {
     /// Creates a new, empty zval.
-    pub(crate) fn new() -> Self {
+    pub(crate) const fn new() -> Self {
         Self {
             value: zend_value {
                 ptr: ptr::null_mut(),
@@ -68,7 +73,23 @@ impl<'a> Zval {
     }
 
     /// Returns the value of the zval if it is a string.
+    ///
+    /// If the zval does not contain a string, the function will check if it contains a
+    /// double or a long, and if so it will convert the value to a [`String`] and return it.
+    /// Don't rely on this logic, as there is potential for this to change to match the output
+    /// of the [`str()`](#method.str) function.
     pub fn string(&self) -> Option<String> {
+        self.str()
+            .map(|s| s.to_string())
+            .or_else(|| self.double().map(|x| x.to_string()))
+    }
+
+    /// Returns the value of the zval if it is a string.
+    ///
+    /// Note that this functions output will not be the same as [`string()`](#method.string), as
+    /// this function does not attempt to convert other types into a [`String`], as it could not
+    /// pass back a [`&str`] in those cases.
+    pub fn str(&'a self) -> Option<&'a str> {
         if self.is_string() {
             // SAFETY: Zend strings have a length that we know we can read.
             // By reading this many bytes we will not run into any issues.
@@ -76,16 +97,14 @@ impl<'a> Zval {
             // We can safely cast our *const c_char into a *const u8 as both
             // only occupy one byte.
             unsafe {
-                let _str = std::str::from_utf8(slice::from_raw_parts(
+                std::str::from_utf8(slice::from_raw_parts(
                     (*self.value.str_).val.as_ptr() as *const u8,
                     (*self.value.str_).len as usize,
                 ))
-                .ok()?;
-
-                Some(_str.to_string())
+                .ok()
             }
         } else {
-            self.double().map(|x| x.to_string())
+            None
         }
     }
 
@@ -97,11 +116,14 @@ impl<'a> Zval {
     ///
     /// There is no way to tell if the data stored in the string is actually of the given type.
     /// The results of this function can also differ from platform-to-platform due to the different
-    /// representation of some types on different platforms. Consult the [`pack`](https://www.php.net/manual/en/function.pack.php)
-    /// function documentation for more details.
-    pub unsafe fn binary<T: Pack>(&self) -> Option<Vec<T>> {
+    /// representation of some types on different platforms. Consult the [`pack`] function
+    /// documentation for more details.
+    ///
+    /// [`pack`]: https://www.php.net/manual/en/function.pack.php
+    pub fn binary<T: Pack>(&self) -> Option<Vec<T>> {
         if self.is_string() {
-            Some(T::unpack_into(self.value.str_.as_ref()?))
+            // SAFETY: Type is string therefore we are able to take a reference.
+            Some(T::unpack_into(unsafe { self.value.str_.as_ref() }?))
         } else {
             None
         }
@@ -119,7 +141,7 @@ impl<'a> Zval {
     }
 
     /// Returns the value of the zval if it is an array.
-    pub fn array(&self) -> Option<ZendHashTable> {
+    pub fn array(&self) -> Option<ZendHashTable<'a>> {
         if self.is_array() {
             unsafe { ZendHashTable::from_ptr(self.value.arr, false) }.ok()
         } else {
@@ -145,7 +167,13 @@ impl<'a> Zval {
         }
     }
 
-    /// Attempts to call the argument as a callable with a list of arguments to pass to the function.
+    /// Returns the value of the zval if it is callable.
+    pub fn callable(&'a self) -> Option<Callable<'a>> {
+        // The Zval is checked if it is callable in the `new` function.
+        Callable::new(self).ok()
+    }
+
+    /// Attempts to call the zval as a callable with a list of arguments to pass to the function.
     /// Note that a thrown exception inside the callable is not detectable, therefore you should
     /// check if the return value is valid rather than unwrapping. Returns a result containing the
     /// return value of the function, or an error.
@@ -156,48 +184,7 @@ impl<'a> Zval {
     ///
     /// * `params` - A list of parameters to call the function with.
     pub fn try_call(&self, params: Vec<&dyn IntoZval>) -> Result<Zval> {
-        let mut retval = Zval::new();
-        let len = params.len();
-        let params = params
-            .into_iter()
-            .map(|val| val.as_zval(false))
-            .collect::<Result<Vec<_>>>()?;
-        let packed = Box::into_raw(params.into_boxed_slice()) as *mut Self;
-        let ptr: *const Self = self;
-
-        if !self.is_callable() {
-            return Err(Error::Callable);
-        }
-
-        let result = unsafe {
-            _call_user_function_impl(
-                std::ptr::null_mut(),
-                ptr as *mut Self,
-                &mut retval,
-                len as _,
-                packed,
-                std::ptr::null_mut(),
-            )
-        };
-
-        // SAFETY: We just boxed this vector, and the `_call_user_function_impl` does not modify the parameters.
-        // We can safely reclaim the memory knowing it will have the same length and size.
-        // If any parameters are zend strings, they must be released.
-        unsafe {
-            let params = Vec::from_raw_parts(packed, len, len);
-
-            for param in params {
-                if param.is_string() {
-                    ext_php_rs_zend_string_release(param.value.str_);
-                }
-            }
-        };
-
-        if result < 0 {
-            Err(Error::Callable)
-        } else {
-            Ok(retval)
-        }
+        self.callable().ok_or(Error::Callable)?.try_call(params)
     }
 
     /// Returns the type of the Zval.
@@ -272,10 +259,7 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     /// * `persistent` - Whether the string should persist between requests.
-    pub fn set_string<S>(&mut self, val: S, persistent: bool) -> Result<()>
-    where
-        S: AsRef<str>,
-    {
+    pub fn set_string(&mut self, val: &str, persistent: bool) -> Result<()> {
         let zend_str = ZendString::new(val, persistent)?;
         self.value.str_ = zend_str.release();
         self.u1.type_info = ZvalTypeFlags::StringEx.bits();
@@ -298,10 +282,7 @@ impl<'a> Zval {
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
-    pub fn set_interned_string<S>(&mut self, val: S) -> Result<()>
-    where
-        S: AsRef<str>,
-    {
+    pub fn set_interned_string(&mut self, val: &str) -> Result<()> {
         let zend_str = ZendString::new_interned(val)?;
         self.value.str_ = zend_str.release();
         self.u1.type_info = ZvalTypeFlags::InternedStringEx.bits();
@@ -369,7 +350,7 @@ impl<'a> Zval {
         self.value.obj = (val as *const ZendObject) as *mut ZendObject;
     }
 
-    /// Sets the value of the zval as an array.
+    /// Sets the value of the zval as an array. Returns nothng in a result on success.
     ///
     /// # Parameters
     ///
@@ -383,6 +364,15 @@ impl<'a> Zval {
     pub(crate) fn set_refcount(&mut self, rc: u32) {
         unsafe { (*self.value.counted).gc.refcount = rc }
     }
+
+    /// Used to drop the Zval but keep the value of the zval intact.
+    ///
+    /// This is important when copying the value of the zval, as the actual value
+    /// will not be copied, but the pointer to the value (string for example) will be
+    /// copied.
+    pub(crate) fn release(mut self) {
+        self.u1.type_info = ZvalTypeFlags::Null.bits();
+    }
 }
 
 impl Debug for Zval {
@@ -413,6 +403,7 @@ impl Debug for Zval {
                 DataType::Callable => field!(self.string()),
                 DataType::ConstantExpression => field!(Option::<()>::None),
                 DataType::Void => field!(Option::<()>::None),
+                DataType::Bool => field!(self.bool()),
             };
         }
 
@@ -420,6 +411,14 @@ impl Debug for Zval {
     }
 }
 
+impl Drop for Zval {
+    fn drop(&mut self) {
+        if self.is_string() {
+            unsafe { ext_php_rs_zend_string_release(self.value.str_) };
+        }
+    }
+}
+
 /// Provides implementations for converting Rust primitive types into PHP zvals. Alternative to the
 /// built-in Rust [`From`] and [`TryFrom`] implementations, allowing the caller to specify whether
 /// the Zval contents will persist between requests.
@@ -430,46 +429,22 @@ pub trait IntoZval {
     /// # Parameters
     ///
     /// * `persistent` - Whether the contents of the Zval will persist between requests.
-    fn as_zval(&self, persistent: bool) -> Result<Zval>;
+    fn as_zval(&self, persistent: bool) -> Result<Zval> {
+        let mut zval = Zval::new();
+        self.set_zval(&mut zval, persistent)?;
+        Ok(zval)
+    }
+
+    /// Sets the content of a pre-existing zval. Returns a result containing nothing if setting
+    /// the content was successful.
+    ///
+    /// # Parameters
+    ///
+    /// * `zv` - The Zval to set the content of.
+    /// * `persistent` - Whether the contents of the Zval will persist between requests.
+    fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()>;
 }
 
-macro_rules! try_from_zval {
-    ($type: ty, $fn: ident) => {
-        impl TryFrom<&Zval> for $type {
-            type Error = Error;
-
-            fn try_from(value: &Zval) -> Result<Self> {
-                match value.$fn() {
-                    Some(v) => match <$type>::try_from(v) {
-                        Ok(v) => Ok(v),
-                        Err(_) => Err(Error::ZvalConversion(value.get_type()?)),
-                    },
-                    _ => Err(Error::ZvalConversion(value.get_type()?)),
-                }
-            }
-        }
-    };
-}
-
-try_from_zval!(i8, long);
-try_from_zval!(i16, long);
-try_from_zval!(i32, long);
-try_from_zval!(i64, long);
-
-try_from_zval!(u8, long);
-try_from_zval!(u16, long);
-try_from_zval!(u32, long);
-try_from_zval!(u64, long);
-
-try_from_zval!(usize, long);
-try_from_zval!(isize, long);
-
-try_from_zval!(f64, double);
-try_from_zval!(bool, bool);
-try_from_zval!(String, string);
-try_from_zval!(ZendHashTable, array);
-
-/// Implements the trait `Into<T>` on Zval for a given type.
 macro_rules! into_zval {
     ($type: ty, $fn: ident) => {
         impl From<$type> for Zval {
@@ -481,10 +456,9 @@ macro_rules! into_zval {
         }
 
         impl IntoZval for $type {
-            fn as_zval(&self, _: bool) -> Result<Zval> {
-                let mut zv = Zval::new();
+            fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
                 zv.$fn(*self);
-                Ok(zv)
+                Ok(())
             }
         }
     };
@@ -493,17 +467,45 @@ macro_rules! into_zval {
 into_zval!(i8, set_long);
 into_zval!(i16, set_long);
 into_zval!(i32, set_long);
-into_zval!(i64, set_long);
 
 into_zval!(u8, set_long);
 into_zval!(u16, set_long);
-into_zval!(u32, set_long);
 
 into_zval!(f32, set_double);
 into_zval!(f64, set_double);
 
 into_zval!(bool, set_bool);
 
+macro_rules! try_into_zval_int {
+    ($type: ty) => {
+        impl TryFrom<$type> for Zval {
+            type Error = Error;
+
+            fn try_from(val: $type) -> Result<Self> {
+                let mut zv = Self::new();
+                let val: ZendLong = val.try_into().map_err(|_| Error::IntegerOverflow)?;
+                zv.set_long(val);
+                Ok(zv)
+            }
+        }
+
+        impl IntoZval for $type {
+            fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+                let val: ZendLong = (*self).try_into().map_err(|_| Error::IntegerOverflow)?;
+                zv.set_long(val);
+                Ok(())
+            }
+        }
+    };
+}
+
+try_into_zval_int!(i64);
+try_into_zval_int!(u32);
+try_into_zval_int!(u64);
+
+try_into_zval_int!(isize);
+try_into_zval_int!(usize);
+
 macro_rules! try_into_zval_str {
     ($type: ty) => {
         impl TryFrom<$type> for Zval {
@@ -511,21 +513,267 @@ macro_rules! try_into_zval_str {
 
             fn try_from(value: $type) -> Result<Self> {
                 let mut zv = Self::new();
-                zv.set_string(value, false)?;
+                zv.set_string(&value, false)?;
                 Ok(zv)
             }
         }
 
         impl IntoZval for $type {
-            fn as_zval(&self, persistent: bool) -> Result<Zval> {
-                let mut zv = Zval::new();
-                zv.set_string(self, persistent)?;
-                Ok(zv)
+            fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()> {
+                zv.set_string(self, persistent)
             }
         }
     };
 }
 
 try_into_zval_str!(String);
-try_into_zval_str!(&String);
 try_into_zval_str!(&str);
+
+impl<T> IntoZval for Option<T>
+where
+    T: IntoZval,
+{
+    fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()> {
+        match self {
+            Some(val) => val.set_zval(zv, persistent),
+            None => {
+                zv.set_null();
+                Ok(())
+            }
+        }
+    }
+}
+
+impl<'a> IntoZval for ZendHashTable<'a> {
+    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+        zv.set_array(self.clone());
+        Ok(())
+    }
+}
+
+impl<T> IntoZval for Vec<T>
+where
+    T: IntoZval,
+{
+    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+        let hm = self
+            .try_into()
+            .map_err(|_| Error::ZvalConversion(DataType::Array))?;
+        zv.set_array(hm);
+        Ok(())
+    }
+}
+
+impl<K, V> IntoZval for HashMap<K, V>
+where
+    K: AsRef<str>,
+    V: IntoZval,
+{
+    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+        let hm = self.try_into()?;
+        zv.set_array(hm);
+        Ok(())
+    }
+}
+
+/// Allows zvals to be converted into Rust types in a fallible way. Reciprocal of the [`IntoZval`]
+/// trait.
+///
+/// This trait requires the [`TryFrom`] trait to be implemented. All this trait does is contain the
+/// type of data that is expected when parsing the value, which is used when parsing arguments.
+pub trait FromZval<'a>: TryFrom<&'a Zval> {
+    /// The corresponding type of the implemented value in PHP.
+    const TYPE: DataType;
+}
+
+impl<'a, T> FromZval<'a> for Option<T>
+where
+    T: FromZval<'a>,
+{
+    const TYPE: DataType = T::TYPE;
+}
+
+// Converting to an option is infallible.
+impl<'a, T> From<&'a Zval> for Option<T>
+where
+    T: FromZval<'a>,
+{
+    fn from(val: &'a Zval) -> Self {
+        val.try_into().ok()
+    }
+}
+
+macro_rules! try_from_zval {
+    ($type: ty, $fn: ident, $dt: ident) => {
+        impl<'a> FromZval<'a> for $type {
+            const TYPE: DataType = DataType::$dt;
+        }
+
+        impl TryFrom<&Zval> for $type {
+            type Error = Error;
+
+            fn try_from(value: &Zval) -> Result<Self> {
+                value
+                    .$fn()
+                    .and_then(|val| val.try_into().ok())
+                    .ok_or(Error::ZvalConversion(value.get_type()?))
+            }
+        }
+
+        impl TryFrom<Zval> for $type {
+            type Error = Error;
+
+            fn try_from(value: Zval) -> Result<Self> {
+                (&value).try_into()
+            }
+        }
+    };
+}
+
+try_from_zval!(i8, long, Long);
+try_from_zval!(i16, long, Long);
+try_from_zval!(i32, long, Long);
+try_from_zval!(i64, long, Long);
+
+try_from_zval!(u8, long, Long);
+try_from_zval!(u16, long, Long);
+try_from_zval!(u32, long, Long);
+try_from_zval!(u64, long, Long);
+
+try_from_zval!(usize, long, Long);
+try_from_zval!(isize, long, Long);
+
+try_from_zval!(f64, double, Double);
+try_from_zval!(bool, bool, Bool);
+try_from_zval!(String, string, String);
+
+impl<'a> FromZval<'a> for f32 {
+    const TYPE: DataType = DataType::Double;
+}
+
+impl<'a> TryFrom<&'a Zval> for f32 {
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        value
+            .double()
+            .map(|v| v as f32)
+            .ok_or(Error::ZvalConversion(value.get_type()?))
+    }
+}
+
+impl<'a> FromZval<'a> for &'a str {
+    const TYPE: DataType = DataType::String;
+}
+
+impl<'a> TryFrom<&'a Zval> for &'a str {
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        value.str().ok_or(Error::ZvalConversion(value.get_type()?))
+    }
+}
+
+impl<'a> FromZval<'a> for ZendHashTable<'a> {
+    const TYPE: DataType = DataType::Array;
+}
+
+impl<'a> TryFrom<&'a Zval> for ZendHashTable<'a> {
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        value
+            .array()
+            .ok_or(Error::ZvalConversion(value.get_type()?))
+    }
+}
+
+impl<'a, T> FromZval<'a> for Vec<T>
+where
+    T: FromZval<'a>,
+{
+    const TYPE: DataType = DataType::Array;
+}
+
+impl<'a, T> TryFrom<&'a Zval> for Vec<T>
+where
+    T: FromZval<'a>,
+{
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        value
+            .array()
+            .ok_or(Error::ZvalConversion(value.get_type()?))?
+            .try_into()
+    }
+}
+
+impl<'a, T> TryFrom<Zval> for Vec<T>
+where
+    T: FromZval<'a>,
+{
+    type Error = Error;
+
+    fn try_from(value: Zval) -> Result<Self> {
+        value
+            .array()
+            .ok_or(Error::ZvalConversion(value.get_type()?))?
+            .try_into()
+    }
+}
+
+impl<'a, T> FromZval<'a> for HashMap<String, T>
+where
+    T: FromZval<'a>,
+{
+    const TYPE: DataType = DataType::Array;
+}
+
+impl<'a, T> TryFrom<&'a Zval> for HashMap<String, T>
+where
+    T: FromZval<'a>,
+{
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        value
+            .array()
+            .ok_or(Error::ZvalConversion(value.get_type()?))?
+            .try_into()
+    }
+}
+
+impl<'a, T> TryFrom<Zval> for HashMap<String, T>
+where
+    T: FromZval<'a>,
+{
+    type Error = Error;
+
+    fn try_from(value: Zval) -> Result<Self> {
+        value
+            .array()
+            .ok_or(Error::ZvalConversion(value.get_type()?))?
+            .try_into()
+    }
+}
+
+impl<'a> FromZval<'a> for Callable<'a> {
+    const TYPE: DataType = DataType::Callable;
+}
+
+impl<'a> TryFrom<&'a Zval> for Callable<'a> {
+    type Error = Error;
+
+    fn try_from(value: &'a Zval) -> Result<Self> {
+        Callable::new(value)
+    }
+}
+
+impl<'a> TryFrom<Zval> for Callable<'a> {
+    type Error = Error;
+
+    fn try_from(value: Zval) -> Result<Self> {
+        Callable::new_owned(value)
+    }
+}
